\name{dynvars}
\alias{dynamic.variable}
\alias{dynamic.bind}
\alias{dynamic.bind.list}

\title{Managing Dynamic State}
\description{
  A simple mechanism for managinh dynamic state in R.
}

\usage{
    dynamic.variable(init)
    dynamic.bind(expr, ...)
    dynamic.bind.list(expr, variables, values)}

\arguments{
  \item{init}{ initial value for the variable }
  \item{expr}{expression to evaluate with dynamic bindings}
  \item{...}{the list of dynamic bindings to establish while evaluating
             the expression} 
  \item{variables}{list of dynamic variables}
  \item{values}{list of values for new dynamic variable bindings}
}

\details{
   Dynamic variables are functions of either zero or one arguments.
   When called with no arguments they return the value of the current
   dynamic binding for the variable.  Calling with one argument
   changes the value of the current dynamic binding to the value of
   the argument.

   \code{dynamic.variable} creates a new dynaic variable and
   initializes its global binding to the value of the \code{init}
   argument.

   \code{dynamic.bind} establishes new dynamic bindings for the
   specified variables and evaluates \code{expr} with these bindings
   in effect.  It then restores the previous bindings and returns the
   value of \code{expr}.  Dynamic variables and the values to use for
   their bindings are specified in \code{name=value} form in the
   \code{...} argument.  The name should be lexically bound to a
   dynamic variable in the environment where \code{dynamic.bind} is
   called.  The values for the new bindings are evaluated in the
   dynamic environment of the call to \code{dynamic.bind}

   \code{dynaimc.bind.list} is a lower level version of
   \code{dynamic.bind} that allows the variables to use to be
   computed.  It is most useful for implementing higher level
   functions like \code{dynamic.bind}.
}

\references{Similar to the \emph{Parameter} mechanism in MzScheme}

\author{Luke Tierney}

\examples{
v <- dynamic.variable(1)
v()
v(2)
v()
dynamic.bind(v(), v = 3)
v()
dynamic.bind({cat("v =", v(),"\n"); v(4); cat("v =", v(), "\n")}, v = 3)
v()
dynamic.bind({cat("v =", v(),"\n"); v(4); cat("v =", v(), "\n")}, v = v())
v()
dynamic.bind.list(v(), list(v), list(3))
v()
}
